@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <limits.h>
+#include <errno.h>
 
 #define WANTSOCKET
 #define WANTARPA
@@ -273,6 +274,29 @@ cmd_allchannels (session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_allchannelslocal (session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	GSList *list = sess_list;
+	server *serv = sess->server;
+
+	if (!*word_eol[2])
+		return FALSE;
+
+	while (list)
+	{
+		sess = list->data;
+		if (sess->type == SESS_CHANNEL && sess->channel[0] &&
+			 sess->server->connected && sess->server == serv)
+		{
+			handle_command (sess, word_eol[2], FALSE);
+		}
+		list = list->next;
+	}
+
+	return TRUE;
+}
+
 static int
 cmd_allservers (struct session *sess, char *tbuf, char *word[],
 					 char *word_eol[])
@@ -391,9 +415,17 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 		if (mask[0] == '~' || mask[0] == '+' ||
 		    mask[0] == '=' || mask[0] == '^' || mask[0] == '-')
 		{
-			safe_strcpy (username, mask+1, sizeof (username));
+			/* the ident is prefixed with something, we replace that sign with an * */
+			safe_strcpy (username+1, mask+1, sizeof (username)-1);
+			username[0] = '*';
+		} else if (at - mask < USERNAMELEN)
+		{
+			/* we just add an * in the begining of the ident */
+			safe_strcpy (username+1, mask, sizeof (username)-1);
+			username[0] = '*';
 		} else
 		{
+			/* ident might be too long, we just ban what it gives and add an * in the end */
 			safe_strcpy (username, mask, sizeof (username));
 		}
 		*at = '@';
@@ -435,11 +467,11 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 				break;
 
 			case 2:
-				sprintf (tbuf, "%s %s *!*%s@%s.*", mode, p2, username, domain);
+				sprintf (tbuf, "%s %s *!%s@%s.*", mode, p2, username, domain);
 				break;
 
 			case 3:
-				sprintf (tbuf, "%s %s *!*%s@%s", mode, p2, username, fullhost);
+				sprintf (tbuf, "%s %s *!%s@%s", mode, p2, username, fullhost);
 				break;
 			}
 		} else
@@ -455,11 +487,11 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 				break;
 
 			case 2:
-				sprintf (tbuf, "%s %s *!*%s@*%s", mode, p2, username, domain);
+				sprintf (tbuf, "%s %s *!%s@*%s", mode, p2, username, domain);
 				break;
 
 			case 3:
-				sprintf (tbuf, "%s %s *!*%s@%s", mode, p2, username, fullhost);
+				sprintf (tbuf, "%s %s *!%s@%s", mode, p2, username, fullhost);
 				break;
 			}
 		}
@@ -526,9 +558,7 @@ cmd_charset (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 
 	if (servlist_check_encoding (word[2 + offset]))
 	{
-		if (serv->encoding)
-			free (serv->encoding);
-		serv->encoding = strdup (word[2 + offset]);
+		server_set_encoding (serv, word[2 + offset]);
 		if (offset < 1)
 			PrintTextf (sess, "Charset changed to: %s\n", word[2 + offset]);
 	} else
@@ -629,14 +659,14 @@ cmd_country (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		/* search? */
 		if (strcmp (code, "-s") == 0)
 		{
-			country_search (word[3], sess, PrintTextf);
+			country_search (word[3], sess, (void *)PrintTextf);
 			return TRUE;
 		}
 
 		/* search, but forgot the -s */
 		if (strchr (code, '*'))
 		{
-			country_search (code, sess, PrintTextf);
+			country_search (code, sess, (void *)PrintTextf);
 			return TRUE;
 		}
 
@@ -707,11 +737,12 @@ cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			}
 			return FALSE;
 		}
-		if (!strcasecmp (type, "CHAT"))
+		if ((!strcasecmp (type, "CHAT")) || (!strcasecmp (type, "PCHAT")))
 		{
 			char *nick = word[3];
+			int passive = (!strcasecmp(type, "PCHAT")) ? 1 : 0;
 			if (*nick)
-				dcc_chat (sess, nick);
+				dcc_chat (sess, nick, passive);
 			return TRUE;
 		}
 		if (!strcasecmp (type, "LIST"))
@@ -940,6 +971,242 @@ cmd_mdeop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+GSList *menu_list = NULL;
+
+static void
+menu_free (menu_entry *me)
+{
+	free (me->path);
+	if (me->label)
+		free (me->label);
+	if (me->cmd)
+		free (me->cmd);
+	if (me->ucmd)
+		free (me->ucmd);
+	free (me);
+}
+
+/* strings equal? but ignore underscores */
+
+int
+menu_streq (const char *s1, const char *s2, int def)
+{
+	/* for separators */
+	if (s1 == NULL && s2 == NULL)
+		return 0;
+	if (s1 == NULL || s2 == NULL)
+		return 1;
+	while (*s1)
+	{
+		if (*s1 == '_')
+			s1++;
+		if (*s2 == '_')
+			s2++;
+		if (*s1 != *s2)
+			return 1;
+		s1++;
+		s2++;
+	}
+	if (!*s2)
+		return 0;
+	return def;
+}
+
+static menu_entry *
+menu_entry_find (char *path, char *label)
+{
+	GSList *list;
+	menu_entry *me;
+
+	list = menu_list;
+	while (list)
+	{
+		me = list->data;
+		if (!strcmp (path, me->path))
+		{
+			if (me->label && label && !strcmp (label, me->label))
+				return me;
+		}
+		list = list->next;
+	}
+	return NULL;
+}
+
+static void
+menu_del_children (char *path, char *label)
+{
+	GSList *list, *next;
+	menu_entry *me;
+	char buf[512];
+
+	if (!label)
+		label = "";
+	if (path[0])
+		snprintf (buf, sizeof (buf), "%s/%s", path, label);
+	else
+		snprintf (buf, sizeof (buf), "%s", label);
+
+	list = menu_list;
+	while (list)
+	{
+		me = list->data;
+		next = list->next;
+		if (!menu_streq (buf, me->path, 0))
+		{
+			menu_list = g_slist_remove (menu_list, me);
+			menu_free (me);
+		}
+		list = next;
+	}
+}
+
+static int
+menu_del (char *path, char *label)
+{
+	GSList *list;
+	menu_entry *me;
+
+	list = menu_list;
+	while (list)
+	{
+		me = list->data;
+		if (!menu_streq (me->label, label, 1) && !menu_streq (me->path, path, 1))
+		{
+			menu_list = g_slist_remove (menu_list, me);
+			fe_menu_del (me);
+			menu_free (me);
+			/* delete this item's children, if any */
+			menu_del_children (path, label);
+			return 1;
+		}
+		list = list->next;
+	}
+
+	return 0;
+}
+
+static void
+menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, int enable, int mod, int key)
+{
+	menu_entry *me;
+
+	/* already exists? */
+	me = menu_entry_find (path, label);
+	if (me)
+	{
+		/* update only */
+		me->state = state;
+		me->enable = enable;
+		fe_menu_update (me);
+		return;
+	}
+
+	me = malloc (sizeof (menu_entry));
+	me->pos = pos;
+	me->state = state;
+	me->enable = enable;
+	me->modifier = mod;
+	me->key = key;
+	me->path = strdup (path);
+	me->label = NULL;
+	me->cmd = NULL;
+	me->ucmd = NULL;
+
+	if (label)
+		me->label = strdup (label);
+	if (cmd)
+		me->cmd = strdup (cmd);
+	if (ucmd)
+		me->ucmd = strdup (ucmd);
+
+	fe_menu_add (me);
+	menu_list = g_slist_append (menu_list, me);
+}
+
+static int
+cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	int idx = 2;
+	int len;
+	int pos = -1;
+	int state;
+	int toggle = FALSE;
+	int enable = TRUE;
+	int key = 0;
+	int mod = 0;
+	char *label;
+
+	if (!word[2][0] || !word[3][0])
+		return FALSE;
+
+	/* -eX enabled or not? */
+	if (word[idx][0] == '-' && word[idx][1] == 'e')
+	{
+		enable = atoi (word[idx] + 2);
+		idx++;
+	}
+
+	/* -k<mod>,<key> key binding */
+	if (word[idx][0] == '-' && word[idx][1] == 'k')
+	{
+		char *comma = strchr (word[idx], ',');
+		if (!comma)
+			return FALSE;
+		mod = atoi (word[idx] + 2);
+		key = atoi (comma + 1);
+		idx++;
+	}
+
+	/* -pX to specify menu position */
+	if (word[idx][0] == '-' && word[idx][1] == 'p')
+	{
+		pos = atoi (word[2] + 2);
+		idx++;
+	}
+
+	/* -tX to specify toggle item with default state */
+	if (word[idx][0] == '-' && word[idx][1] == 't')
+	{
+		state = atoi (word[idx] + 2);
+		idx++;
+		toggle = TRUE;
+	}
+
+	/* the path */
+	path_part (word[idx+1], tbuf, 512);
+	len = strlen (tbuf);
+	if (len)
+		tbuf[len - 1] = 0;
+
+	/* the name of the item */
+	label = file_part (word[idx + 1]);
+	if (label[0] == '-' && label[1] == 0)
+		label = NULL;	/* separator */
+
+	if (!strcasecmp (word[idx], "ADD"))
+	{
+		if (toggle)
+		{
+			menu_add (tbuf, label, word[idx + 2], word[idx + 3], pos, state, enable, mod, key);
+		} else
+		{
+			if (word[idx + 2][0])
+				menu_add (tbuf, label, word[idx + 2], NULL, pos, 0, enable, mod, key);
+			else
+				menu_add (tbuf, label, NULL, NULL, pos, 0, enable, mod, key);
+		}
+		return TRUE;
+	}
+
+	if (!strcasecmp (word[idx], "DEL"))
+	{
+		menu_del (tbuf, label);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 mkick_cb (struct User *user, multidata *data)
 {
@@ -1282,7 +1549,7 @@ exec_data (GIOChannel *source, GIOCondition condition, struct nbexec *s)
 	}
 	else
 		readpos = buf = malloc(2050);
-	
+
 	rd = read (sok, readpos, 2048);
 	if (rd < 1)
 	{
@@ -1329,7 +1596,7 @@ exec_data (GIOChannel *source, GIOCondition condition, struct nbexec *s)
 		else
 			PrintText (s->sess, buf);
 	}
-	
+
 	free(buf);
 	return TRUE;
 }
@@ -1342,6 +1609,7 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	int fds[2], pid = 0;
 	struct nbexec *s;
 	int shell = TRUE;
+	int fd;
 
 	if (*cmd)
 	{
@@ -1411,6 +1679,8 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			dup2 (fds[1], 2);
 			/* Also copy it to stdin so we can write to it */
 			dup2 (fds[1], 0);
+			/* Now close all open file descriptors except stdin, stdout and stderr */
+			for (fd = 3; fd < 1024; fd++) close(fd);
 			/* Now we call /bin/sh to run our cmd ; made it more friendly -DC1 */
 			if (shell)
 			{
@@ -1584,6 +1854,16 @@ cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	} else if (!strcasecmp (word[2], "ICONIFY"))
 	{
 		fe_ctrl_gui (sess, 5, 0);
+	} else if (!strcasecmp (word[2], "MENU"))
+	{
+		if (!strcasecmp (word[3], "TOGGLE"))
+			fe_ctrl_gui (sess, 6, 0);
+		else
+			return FALSE;
+	} else if (!strcasecmp (word[2], "MSGBOX"))
+	{
+		fe_message (word[3], FALSE);
+		return TRUE;
 	} else
 	{
 		return FALSE;
@@ -1592,11 +1872,71 @@ cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+typedef struct
+{
+	int longfmt;
+	int i, t;
+	char *buf;
+} help_list;
+
+static void
+show_help_line (session *sess, help_list *hl, char *name, char *usage)
+{
+	int j, len, max;
+	char *p;
+
+	if (hl->longfmt)	/* long format for /HELP -l */
+	{
+		if (!usage || usage[0] == 0)
+			PrintTextf (sess, "   \0034%s\003 :\n", name);
+		else
+			PrintTextf (sess, "   \0034%s\003 : %s\n", name, _(usage));
+		return;
+	}
+
+	/* append the name into buffer, but convert to uppercase */
+	len = strlen (hl->buf);
+	p = name;
+	while (*p)
+	{
+		hl->buf[len] = toupper (*p);
+		len++;
+		p++;
+	}
+	hl->buf[len] = 0;
+
+	hl->t++;
+	if (hl->t == 5)
+	{
+		hl->t = 0;
+		strcat (hl->buf, "\n");
+		PrintText (sess, hl->buf);
+		hl->buf[0] = ' ';
+		hl->buf[1] = ' ';
+		hl->buf[2] = 0;
+	} else
+	{
+		/* append some spaces after the command name */
+		max = strlen (name);
+		if (max < 10)
+		{
+			max = 10 - max;
+			for (j = 0; j < max; j++)
+			{
+				hl->buf[len] = ' ';
+				len++;
+				hl->buf[len] = 0;
+			}
+		}
+	}
+}
+
 static int
 cmd_help (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	int i = 0, longfmt = 0;
 	char *helpcmd = "";
+	GSList *list;
 
 	if (tbuf)
 		helpcmd = word[2];
@@ -1609,79 +1949,70 @@ cmd_help (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	} else
 	{
 		struct popup *pop;
-		GSList *list = command_list;
 		char *buf = malloc (4096);
-		int t = 1, j;
-		strcpy (buf, _("\nCommands Available:\n\n  "));
-		if (longfmt)
+		help_list hl;
+
+		hl.longfmt = longfmt;
+		hl.buf = buf;
+
+		PrintTextf (sess, "\n%s\n\n", _("Commands Available:"));
+		buf[0] = ' ';
+		buf[1] = ' ';
+		buf[2] = 0;
+		hl.t = 0;
+		hl.i = 0;
+		while (xc_cmds[i].name)
 		{
-			while (1)
-			{
-				if (!xc_cmds[i].name)
-					break;
-				if (!xc_cmds[i].help || *xc_cmds[i].help == '\0')
-					snprintf (buf, 4096, "   \0034%s\003 :\n", xc_cmds[i].name);
-				else
-					snprintf (buf, 4096, "   \0034%s\003 : %s\n", xc_cmds[i].name,
-								 _(xc_cmds[i].help));
-				PrintText (sess, buf);
-				i++;
-			}
-			buf[0] = 0;
-		} else
-		{
-			while (1)
-			{
-				if (!xc_cmds[i].name)
-					break;
-				strcat (buf, xc_cmds[i].name);
-				t++;
-				if (t == 6)
-				{
-					t = 1;
-					strcat (buf, "\n  ");
-				} else
-					for (j = 0; j < (10 - strlen (xc_cmds[i].name)); j++)
-						strcat (buf, " ");
-				i++;
-			}
+			show_help_line (sess, &hl, xc_cmds[i].name, xc_cmds[i].help);
+			i++;
 		}
-		strcat (buf,
-				  _("\n\nType /HELP <command> for more information, or /HELP -l\n\n"));
-		strcat (buf, _("User defined commands:\n\n  "));
-		t = 1;
+		strcat (buf, "\n");
+		PrintText (sess, buf);
+
+		PrintTextf (sess, "\n%s\n\n", _("User defined commands:"));
+		buf[0] = ' ';
+		buf[1] = ' ';
+		buf[2] = 0;
+		hl.t = 0;
+		hl.i = 0;
+		list = command_list;
 		while (list)
 		{
-			pop = (struct popup *) list->data;
-			strcat (buf, pop->name);
-			t++;
-			if (t == 6)
-			{
-				t = 1;
-				strcat (buf, "\n");
-				PrintText (sess, buf);
-				strcpy (buf, "  ");
-			} else
-			{
-				if (strlen (pop->name) < 10)
-				{
-					for (j = 0; j < (10 - strlen (pop->name)); j++)
-						strcat (buf, " ");
-				}
-			}
+			pop = list->data;
+			show_help_line (sess, &hl, pop->name, pop->cmd);
 			list = list->next;
 		}
 		strcat (buf, "\n");
 		PrintText (sess, buf);
+
+		PrintTextf (sess, "\n%s\n\n", _("Plugin defined commands:"));
+		buf[0] = ' ';
+		buf[1] = ' ';
+		buf[2] = 0;
+		hl.t = 0;
+		hl.i = 0;
+		plugin_command_foreach (sess, &hl, (void *)show_help_line);
+		strcat (buf, "\n");
+		PrintText (sess, buf);
 		free (buf);
 
-		PrintText (sess, "\nPlugin defined commands:\n\n");
-		plugin_show_help (sess, NULL);
-
+		PrintTextf (sess, "\n%s\n\n", _("Type /HELP <command> for more information, or /HELP -l"));
 	}
 	return TRUE;
 }
 
+static int
+cmd_id (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (word[2][0])
+	{
+		sess->server->p_nickserv (sess->server, word[2]);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 cmd_ignore (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1878,23 +2209,34 @@ cmd_load (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	char *error, *arg, *nl, *file;
 	int len;
 
+	if (!word[2][0])
+		return FALSE;
+
 	if (strcmp (word[2], "-e") == 0)
 	{
 		file = expand_homedir (word[3]);
 		fp = fopen (file, "r");
-		free (file);
-		if (fp)
+		if (!fp)
 		{
-			tbuf[1024] = 0;
-			while (fgets (tbuf, 1024, fp))
-			{
-				nl = strchr (tbuf, '\n');
-				if (nl)
-					*nl = 0;
-				handle_command (sess, tbuf, TRUE);
-			}
-			fclose (fp);
+			PrintTextf (sess, "Cannot access %s\n", file);
+			PrintText (sess, errorstring (errno));
+			free (file);
+			return TRUE;
 		}
+		free (file);
+
+		tbuf[1024] = 0;
+		while (fgets (tbuf, 1024, fp))
+		{
+			nl = strchr (tbuf, '\n');
+			if (nl)
+				*nl = 0;
+			if (tbuf[0] == prefs.cmdchar[0])
+				handle_command (sess, tbuf + 1, TRUE);
+			else
+				handle_command (sess, tbuf, TRUE);
+		}
+		fclose (fp);
 		return TRUE;
 	}
 
@@ -1973,13 +2315,13 @@ cmd_mode (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	/* +channel channels are dying, let those servers whine about modes.
 	 * return info about current channel if available and no info is given */
-	if ((*word[2] == '+') || (*word[2] == 0) || (!is_channel(sess->server, word[2]) && 
+	if ((*word[2] == '+') || (*word[2] == 0) || (!is_channel(sess->server, word[2]) &&
 				!(rfc_casecmp(sess->server->nick, word[2]) == 0)))
 	{
 		if(sess->channel[0] == 0)
 			return FALSE;
 		sess->server->p_mode (sess->server, sess->channel, word_eol[2]);
-	}	
+	}
 	else
 		sess->server->p_mode (sess->server, word[2], word_eol[3]);
 	return TRUE;
@@ -2242,8 +2584,8 @@ cmd_reconnect (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				serv->auto_reconnect (serv, TRUE, -1);
 			list = list->next;
 		}
-	}	
-	/* If it isn't "ALL" and there is something 
+	}
+	/* If it isn't "ALL" and there is something
 	there it *should* be a server they are trying to connect to*/
 	else if (*word[2])
 	{
@@ -2270,7 +2612,7 @@ cmd_reconnect (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	else
 	{
 		serv->auto_reconnect (serv, TRUE, -1);
-	}	
+	}
 	prefs.recon_delay = tmp;
 
 	return TRUE;
@@ -2559,6 +2901,28 @@ userlist_cb (struct User *user, session *sess)
 	return TRUE;
 }
 
+static int
+cmd_uselect (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	int idx = 2;
+	int clear = TRUE;
+	int scroll = FALSE;
+
+	if (strcmp (word[2], "-a") == 0)	/* ADD (don't clear selections) */
+	{
+		clear = FALSE;
+		idx++;
+	}
+	if (strcmp (word[idx], "-s") == 0)	/* SCROLL TO */
+	{
+		scroll = TRUE;
+		idx++;
+	}
+	/* always valid, no args means clear the selection list */
+	fe_uselect (sess, word + idx, clear, scroll);
+	return TRUE;
+}
+
 static int
 cmd_userlist (struct session *sess, char *tbuf, char *word[],
 				  char *word_eol[])
@@ -2679,171 +3043,179 @@ cmd_voice (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 
 /* *MUST* be kept perfectly sorted for the bsearch to work */
 const struct commands xc_cmds[] = {
-	{"ADDBUTTON", cmd_addbutton, 0, 0,
+	{"ADDBUTTON", cmd_addbutton, 0, 0, 1,
 	 N_("ADDBUTTON <name> <action>, adds a button under the user-list")},
-	{"ALLCHAN", cmd_allchannels, 0, 0,
+	{"ALLCHAN", cmd_allchannels, 0, 0, 1,
 	 N_("ALLCHAN <cmd>, sends a command to all channels you're in")},
-	{"ALLSERV", cmd_allservers, 0, 0,
+	{"ALLCHANL", cmd_allchannelslocal, 0, 0, 1,
+	 N_("ALLCHANL <cmd>, sends a command to all channels you're in")},
+	{"ALLSERV", cmd_allservers, 0, 0, 1,
 	 N_("ALLSERV <cmd>, sends a command to all servers you're in")},
-	{"AWAY", cmd_away, 1, 0, N_("AWAY [<reason>], sets you away")},
-	{"BAN", cmd_ban, 1, 1,
+	{"AWAY", cmd_away, 1, 0, 1, N_("AWAY [<reason>], sets you away")},
+	{"BAN", cmd_ban, 1, 1, 1,
 	 N_("BAN <mask> [<bantype>], bans everyone matching the mask from the current channel. If they are already on the channel this doesn't kick them (needs chanop)")},
-	{"CHARSET", cmd_charset, 0, 0, 0},
-	{"CLEAR", cmd_clear, 0, 0, N_("CLEAR, Clears the current text window")},
-	{"CLOSE", cmd_close, 0, 0, N_("CLOSE, Closes the current window/tab")},
+	{"CHARSET", cmd_charset, 0, 0, 1, 0},
+	{"CLEAR", cmd_clear, 0, 0, 1, N_("CLEAR, Clears the current text window")},
+	{"CLOSE", cmd_close, 0, 0, 1, N_("CLOSE, Closes the current window/tab")},
 
-	{"COUNTRY", cmd_country, 0, 0,
+	{"COUNTRY", cmd_country, 0, 0, 1,
 	 N_("COUNTRY [-s] <code|wildcard>, finds a country code, eg: au = australia")},
-	{"CTCP", cmd_ctcp, 1, 0,
+	{"CTCP", cmd_ctcp, 1, 0, 1,
 	 N_("CTCP <nick> <message>, send the CTCP message to nick, common messages are VERSION and USERINFO")},
-	{"CYCLE", cmd_cycle, 1, 1,
+	{"CYCLE", cmd_cycle, 1, 1, 1,
 	 N_("CYCLE, parts current channel and immediately rejoins")},
-	{"DCC", cmd_dcc, 0, 0,
+	{"DCC", cmd_dcc, 0, 0, 1,
 	 N_("\n"
-	 "DCC GET <nick>                     - accept an offered file\n"
-	 "DCC SEND [-maxcps=#] <nick> [file] - send a file to someone\n"
-	 "DCC LIST                           - show DCC list\n"
-	 "DCC CHAT <nick>                    - offer DCC CHAT to someone\n"
+	 "DCC GET <nick>                      - accept an offered file\n"
+	 "DCC SEND [-maxcps=#] <nick> [file]  - send a file to someone\n"
+	 "DCC PSEND [-maxcps=#] <nick> [file] - send a file using passive mode\n"
+	 "DCC LIST                            - show DCC list\n"
+	 "DCC CHAT <nick>                     - offer DCC CHAT to someone\n"
+	 "DCC PCHAT <nick>                    - offer DCC CHAT using passive mode\n"
 	 "DCC CLOSE <type> <nick> <file>         example:\n"
 	 "         /dcc close send johnsmith file.tar.gz")},
-	{"DEBUG", cmd_debug, 0, 0, 0},
+	{"DEBUG", cmd_debug, 0, 0, 1, 0},
 
-	{"DEHOP", cmd_dehop, 1, 1,
+	{"DEHOP", cmd_dehop, 1, 1, 1,
 	 N_("DEHOP <nick>, removes chanhalf-op status from the nick on the current channel (needs chanop)")},
-	{"DELBUTTON", cmd_delbutton, 0, 0,
+	{"DELBUTTON", cmd_delbutton, 0, 0, 1,
 	 N_("DELBUTTON <name>, deletes a button from under the user-list")},
-	{"DEOP", cmd_deop, 1, 1,
+	{"DEOP", cmd_deop, 1, 1, 1,
 	 N_("DEOP <nick>, removes chanop status from the nick on the current channel (needs chanop)")},
-	{"DEVOICE", cmd_devoice, 1, 1,
+	{"DEVOICE", cmd_devoice, 1, 1, 1,
 	 N_("DEVOICE <nick>, removes voice status from the nick on the current channel (needs chanop)")},
-	{"DISCON", cmd_discon, 0, 0, N_("DISCON, Disconnects from server")},
-	{"DNS", cmd_dns, 0, 0, N_("DNS <nick|host|ip>, Finds a users IP number")},
-	{"ECHO", cmd_echo, 0, 0, N_("ECHO <text>, Prints text locally")},
+	{"DISCON", cmd_discon, 0, 0, 1, N_("DISCON, Disconnects from server")},
+	{"DNS", cmd_dns, 0, 0, 1, N_("DNS <nick|host|ip>, Finds a users IP number")},
+	{"ECHO", cmd_echo, 0, 0, 1, N_("ECHO <text>, Prints text locally")},
 #ifndef WIN32
-	{"EXEC", cmd_exec, 0, 0,
+	{"EXEC", cmd_exec, 0, 0, 1,
 	 N_("EXEC [-o] <command>, runs the command. If -o flag is used then output is sent to current channel, else is printed to current text box")},
 #ifndef __EMX__
-	{"EXECCONT", cmd_execc, 0, 0, N_("EXECCONT, sends the process SIGCONT")},
+	{"EXECCONT", cmd_execc, 0, 0, 1, N_("EXECCONT, sends the process SIGCONT")},
 #endif
-	{"EXECKILL", cmd_execk, 0, 0,
+	{"EXECKILL", cmd_execk, 0, 0, 1,
 	 N_("EXECKILL [-9], kills a running exec in the current session. If -9 is given the process is SIGKILL'ed")},
 #ifndef __EMX__
-	{"EXECSTOP", cmd_execs, 0, 0, N_("EXECSTOP, sends the process SIGSTOP")},
-	{"EXECWRITE", cmd_execw, 0, 0, N_("EXECWRITE, sends data to the processes stdin")},
+	{"EXECSTOP", cmd_execs, 0, 0, 1, N_("EXECSTOP, sends the process SIGSTOP")},
+	{"EXECWRITE", cmd_execw, 0, 0, 1, N_("EXECWRITE, sends data to the processes stdin")},
 #endif
 #endif
-	{"FLUSHQ", cmd_flushq, 0, 0,
+	{"FLUSHQ", cmd_flushq, 0, 0, 1,
 	 N_("FLUSHQ, flushes the current server's send queue")},
-	{"GATE", cmd_gate, 0, 0,
+	{"GATE", cmd_gate, 0, 0, 1,
 	 N_("GATE <host> [<port>], proxies through a host, port defaults to 23")},
-	{"GETINT", cmd_getint, 0, 0, "GETINT <default> <command> <prompt>"},
-	{"GETSTR", cmd_getstr, 0, 0, "GETSTR <default> <command> <prompt>"},
-	{"GUI", cmd_gui, 0, 0, "GUI [SHOW|HIDE|FOCUS|FLASH|ICONIFY|COLOR <n>]"},
-	{"HELP", cmd_help, 0, 0, 0},
-	{"HOP", cmd_hop, 1, 1,
+	{"GETINT", cmd_getint, 0, 0, 1, "GETINT <default> <command> <prompt>"},
+	{"GETSTR", cmd_getstr, 0, 0, 1, "GETSTR <default> <command> <prompt>"},
+	{"GUI", cmd_gui, 0, 0, 1, "GUI [SHOW|HIDE|FOCUS|FLASH|ICONIFY|MENU TOGGLE|COLOR <n>|MSGBOX <text>]"},
+	{"HELP", cmd_help, 0, 0, 1, 0},
+	{"HOP", cmd_hop, 1, 1, 1,
 	 N_("HOP <nick>, gives chanhalf-op status to the nick (needs chanop)")},
-	{"IGNORE", cmd_ignore, 0, 0,
+	{"ID", cmd_id, 1, 0, 1, N_("ID <password>, identifies yourself to nickserv")},
+	{"IGNORE", cmd_ignore, 0, 0, 1,
 	 N_("IGNORE <mask> <types..> <options..>\n"
 	 "    mask - host mask to ignore, eg: *!*@*.aol.com\n"
 	 "    types - types of data to ignore, one or all of:\n"
 	 "            PRIV, CHAN, NOTI, CTCP, DCC, INVI, ALL\n"
 	 "    options - NOSAVE, QUIET")},
 
-	{"INVITE", cmd_invite, 1, 0,
+	{"INVITE", cmd_invite, 1, 0, 1,
 	 N_("INVITE <nick> [<channel>], invites someone to a channel, by default the current channel (needs chanop)")},
-	{"JOIN", cmd_join, 1, 0, N_("JOIN <channel>, joins the channel")},
-	{"KICK", cmd_kick, 1, 1,
+	{"JOIN", cmd_join, 1, 0, 0, N_("JOIN <channel>, joins the channel")},
+	{"KICK", cmd_kick, 1, 1, 1,
 	 N_("KICK <nick>, kicks the nick from the current channel (needs chanop)")},
-	{"KICKBAN", cmd_kickban, 1, 1,
+	{"KICKBAN", cmd_kickban, 1, 1, 1,
 	 N_("KICKBAN <nick>, bans then kicks the nick from the current channel (needs chanop)")},
-	{"KILLALL", cmd_killall, 0, 0, "KILLALL, immediately exit"},
-	{"LAGCHECK", cmd_lagcheck, 0, 0,
+	{"KILLALL", cmd_killall, 0, 0, 1, "KILLALL, immediately exit"},
+	{"LAGCHECK", cmd_lagcheck, 0, 0, 1,
 	 N_("LAGCHECK, forces a new lag check")},
-	{"LASTLOG", cmd_lastlog, 0, 0,
+	{"LASTLOG", cmd_lastlog, 0, 0, 1,
 	 N_("LASTLOG <string>, searches for a string in the buffer")},
-	{"LIST", cmd_list, 1, 0, 0},
-	{"LOAD", cmd_load, 0, 0, N_("LOAD <file>, loads a plugin or script")},
+	{"LIST", cmd_list, 1, 0, 1, 0},
+	{"LOAD", cmd_load, 0, 0, 1, N_("LOAD [-e] <file>, loads a plugin or script")},
 
-	{"MDEHOP", cmd_mdehop, 1, 1,
+	{"MDEHOP", cmd_mdehop, 1, 1, 1,
 	 N_("MDEHOP, Mass deop's all chanhalf-ops in the current channel (needs chanop)")},
-	{"MDEOP", cmd_mdeop, 1, 1,
+	{"MDEOP", cmd_mdeop, 1, 1, 1,
 	 N_("MDEOP, Mass deop's all chanops in the current channel (needs chanop)")},
-	{"ME", cmd_me, 0, 0,
+	{"ME", cmd_me, 0, 0, 1,
 	 N_("ME <action>, sends the action to the current channel (actions are written in the 3rd person, like /me jumps)")},
-	{"MKICK", cmd_mkick, 1, 1,
+	{"MENU", cmd_menu, 0, 0, 1, "MENU [-eX] [-k<mod>,<key>] [-pX] [-tX] {ADD|DEL} <path> [command] [unselect command]"},
+	{"MKICK", cmd_mkick, 1, 1, 1,
 	 N_("MKICK, Mass kicks everyone except you in the current channel (needs chanop)")},
-	{"MODE", cmd_mode, 1, 0, 0},
-	{"MOP", cmd_mop, 1, 1,
+	{"MODE", cmd_mode, 1, 0, 1, 0},
+	{"MOP", cmd_mop, 1, 1, 1,
 	 N_("MOP, Mass op's all users in the current channel (needs chanop)")},
-	{"MSG", cmd_msg, 0, 0, N_("MSG <nick> <message>, sends a private message")},
+	{"MSG", cmd_msg, 0, 0, 1, N_("MSG <nick> <message>, sends a private message")},
 
-	{"NAMES", cmd_names, 1, 0,
+	{"NAMES", cmd_names, 1, 0, 1,
 	 N_("NAMES, Lists the nicks on the current channel")},
-	{"NCTCP", cmd_nctcp, 1, 0,
+	{"NCTCP", cmd_nctcp, 1, 0, 1,
 	 N_("NCTCP <nick> <message>, Sends a CTCP notice")},
-	{"NEWSERVER", cmd_newserver, 0, 0, N_("NEWSERVER <hostname> [<port>]")},
-	{"NICK", cmd_nick, 0, 0, N_("NICK <nickname>, sets your nick")},
+	{"NEWSERVER", cmd_newserver, 0, 0, 1, N_("NEWSERVER <hostname> [<port>]")},
+	{"NICK", cmd_nick, 0, 0, 1, N_("NICK <nickname>, sets your nick")},
 
-	{"NOTICE", cmd_notice, 1, 0,
+	{"NOTICE", cmd_notice, 1, 0, 1,
 	 N_("NOTICE <nick/channel> <message>, sends a notice. Notices are a type of message that should be auto reacted to")},
-	{"NOTIFY", cmd_notify, 0, 0,
+	{"NOTIFY", cmd_notify, 0, 0, 1,
 	 N_("NOTIFY [<nick>], lists your notify list or adds someone to it")},
-	{"OP", cmd_op, 1, 1,
+	{"OP", cmd_op, 1, 1, 1,
 	 N_("OP <nick>, gives chanop status to the nick (needs chanop)")},
-	{"PART", cmd_part, 1, 1,
+	{"PART", cmd_part, 1, 1, 1,
 	 N_("PART [<channel>] [<reason>], leaves the channel, by default the current one")},
-	{"PING", cmd_ping, 1, 0,
+	{"PING", cmd_ping, 1, 0, 1,
 	 N_("PING <nick | channel>, CTCP pings nick or channel")},
-	{"QUERY", cmd_query, 0, 0,
+	{"QUERY", cmd_query, 0, 0, 1,
 	 N_("QUERY <nick>, opens up a new privmsg window to someone")},
-	{"QUIT", cmd_quit, 0, 0,
+	{"QUIT", cmd_quit, 0, 0, 1,
 	 N_("QUIT [<reason>], disconnects from the current server")},
-	{"QUOTE", cmd_quote, 1, 0,
+	{"QUOTE", cmd_quote, 1, 0, 1,
 	 N_("QUOTE <text>, sends the text in raw form to the server")},
 #ifdef USE_OPENSSL
-	{"RECONNECT", cmd_reconnect, 0, 0,
-	 N_("RECONNECT [-ssl] [<host>] [<port>] [<password>], Can be called just as /RECONNECT to reconnect to the current server or with /RECONNECT ALL to reconnect to all the open servers")}, 
-#else 
-	{"RECONNECT", cmd_reconnect, 0, 0,
+	{"RECONNECT", cmd_reconnect, 0, 0, 1,
+	 N_("RECONNECT [-ssl] [<host>] [<port>] [<password>], Can be called just as /RECONNECT to reconnect to the current server or with /RECONNECT ALL to reconnect to all the open servers")},
+#else
+	{"RECONNECT", cmd_reconnect, 0, 0, 1,
 	 N_("RECONNECT [<host>] [<port>] [<password>], Can be called just as /RECONNECT to reconnect to the current server or with /RECONNECT ALL to reconnect to all the open servers")},
 #endif
-	{"RECV", cmd_recv, 1, 0, N_("RECV <text>, send raw data to xchat, as if it was received from the irc server")},
+	{"RECV", cmd_recv, 1, 0, 1, N_("RECV <text>, send raw data to xchat, as if it was received from the irc server")},
 
-	{"SAY", cmd_say, 0, 0,
+	{"SAY", cmd_say, 0, 0, 1,
 	 N_("SAY <text>, sends the text to the object in the current window")},
 #ifdef USE_OPENSSL
-	{"SERVCHAN", cmd_servchan, 0, 0,
+	{"SERVCHAN", cmd_servchan, 0, 0, 1,
 	 N_("SERVCHAN [-ssl] <host> <port> <channel>, connects and joins a channel")},
 #else
-	{"SERVCHAN", cmd_servchan, 0, 0,
+	{"SERVCHAN", cmd_servchan, 0, 0, 1,
 	 N_("SERVCHAN <host> <port> <channel>, connects and joins a channel")},
 #endif
 #ifdef USE_OPENSSL
-	{"SERVER", cmd_server, 0, 0,
+	{"SERVER", cmd_server, 0, 0, 1,
 	 N_("SERVER [-ssl] <host> [<port>] [<password>], connects to a server, the default port is 6667 for normal connections, and 9999 for ssl connections")},
 #else
-	{"SERVER", cmd_server, 0, 0,
+	{"SERVER", cmd_server, 0, 0, 1,
 	 N_("SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667")},
 #endif
-	{"SET", cmd_set, 0, 0, N_("SET [-quiet] <variable> [<value>]")},
-	{"SETCURSOR", cmd_setcursor, 0, 0, N_("SETCURSOR [-|+]<position>")},
-	{"SETTAB", cmd_settab, 0, 0, 0},
-	{"SETTEXT", cmd_settext, 0, 0, 0},
-	{"TOPIC", cmd_topic, 1, 1,
+	{"SET", cmd_set, 0, 0, 1, N_("SET [-quiet] <variable> [<value>]")},
+	{"SETCURSOR", cmd_setcursor, 0, 0, 1, N_("SETCURSOR [-|+]<position>")},
+	{"SETTAB", cmd_settab, 0, 0, 1, 0},
+	{"SETTEXT", cmd_settext, 0, 0, 1, 0},
+	{"TOPIC", cmd_topic, 1, 1, 1,
 	 N_("TOPIC [<topic>], sets the topic if one is given, else shows the current topic")},
-	{"UNBAN", cmd_unban, 1, 1,
+	{"UNBAN", cmd_unban, 1, 1, 1,
 	 N_("UNBAN <mask> [<mask>...], unbans the specified masks.")},
-	{"UNIGNORE", cmd_unignore, 0, 0, N_("UNIGNORE <mask> [QUIET]")},
-	{"UNLOAD", cmd_unload, 0, 0, N_("UNLOAD <name>, unloads a plugin or script")},
-	{"URL", cmd_url, 0, 0, N_("URL <url>, opens a URL in your browser")},
-	{"USERLIST", cmd_userlist, 1, 1, 0},
-	{"VOICE", cmd_voice, 1, 1,
+	{"UNIGNORE", cmd_unignore, 0, 0, 1, N_("UNIGNORE <mask> [QUIET]")},
+	{"UNLOAD", cmd_unload, 0, 0, 1, N_("UNLOAD <name>, unloads a plugin or script")},
+	{"URL", cmd_url, 0, 0, 1, N_("URL <url>, opens a URL in your browser")},
+	{"USELECT", cmd_uselect, 0, 1, 0,
+	 N_("USELECT [-a] [-s] <nick1> <nick2> etc, highlights nick(s) in channel userlist")},
+	{"USERLIST", cmd_userlist, 1, 1, 1, 0},
+	{"VOICE", cmd_voice, 1, 1, 1,
 	 N_("VOICE <nick>, gives voice status to someone (needs chanop)")},
-	{"WALLCHAN", cmd_wallchan, 1, 1,
+	{"WALLCHAN", cmd_wallchan, 1, 1, 1,
 	 N_("WALLCHAN <message>, writes the message to all channels")},
-	{"WALLCHOP", cmd_wallchop, 1, 1,
+	{"WALLCHOP", cmd_wallchop, 1, 1, 1,
 	 N_("WALLCHOP <message>, sends the message to all chanops on the current channel")},
-	{0, 0, 0, 0, 0}
+	{0, 0, 0, 0, 0, 0}
 };
 
 
@@ -3393,6 +3765,10 @@ handle_command (session *sess, char *cmd, int check_spch)
 
 	/* split the text into words and word_eol */
 	process_data_init (pdibuf, cmd, word, word_eol, TRUE);
+	int_cmd = find_internal_command (word[1]);
+	/* redo it without quotes processing, for some commands like /JOIN */
+	if (int_cmd && !int_cmd->handle_quotes)
+		process_data_init (pdibuf, cmd, word, word_eol, FALSE);
 
 	if (check_spch && prefs.perc_color)
 		check_special_chars (cmd, prefs.perc_ascii);
