@@ -228,8 +228,9 @@ void
 tcp_sendf (server *serv, char *fmt, ...)
 {
 	va_list args;
-	/* keep this buffer in BSS */
-	static char send_buf[520];	/* good code hey (no it's not overflowable) */
+	/* keep this buffer in BSS. Converting UTF-8 to ISO-8895-x might make the
+      string shorter, so allow alot more than 512 for now. */
+	static char send_buf[1540];	/* good code hey (no it's not overflowable) */
 	int len;
 
 	va_start (args, fmt);
@@ -267,8 +268,8 @@ server_inline (server *serv, char *line, int len)
 	/* Checks whether we're set to use UTF-8 charset */
 	if ((serv->encoding == NULL && prefs.utf8_locale) ||
 	    (serv->encoding != NULL &&
-		 (strcasecmp (serv->encoding, "utf8") == 0 ||
-		  strcasecmp (serv->encoding, "utf-8") == 0)))
+		 (strcasecmp (serv->encoding, "UTF8") == 0 ||
+		  strcasecmp (serv->encoding, "UTF-8") == 0)))
 	{
 		/* The user has the UTF-8 charset set, either via /charset
 		command or from his UTF-8 locale. Thus, we first try the
@@ -306,6 +307,11 @@ server_inline (server *serv, char *line, int len)
 			conv_line[len] = 0;
 			conv_len = len;
 
+			/* if CP1255, convert it with the NUL terminator.
+				Works around SF bug #1122089 */
+			if (serv->using_cp1255)
+				conv_len++;
+
 			do
 			{
 				err = NULL;
@@ -333,6 +339,8 @@ server_inline (server *serv, char *line, int len)
 			{
 				line = utf_line_allocated;
 				len = utf_len;
+				if (serv->using_cp1255 && len > 0)
+					len--;
 			}
 			else
 			{
@@ -343,7 +351,7 @@ server_inline (server *serv, char *line, int len)
 	}
 
 	fe_add_rawlog (serv, line, len, FALSE);
-	url_check (line);
+	url_check_line (line, len);
 
 	/* let proto-irc.c handle it */
 	serv->p_inline (serv, line, len);
@@ -414,7 +422,7 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 				serv->linebuf[serv->pos] = 0;
 				if (prefs.stripcolor)
 				{
-					temp = strip_color (serv->linebuf);
+					temp = strip_color (serv->linebuf, -1, 1, 1);
 					server_inline (serv, temp, strlen (temp));
 					free (temp);
 				} else
@@ -452,10 +460,12 @@ server_connected (server * serv)
 		if (serv->network)
 		{
 			serv->p_login (serv,
-								(((ircnet *)serv->network)->user) ?
+								(!(((ircnet *)serv->network)->flags & FLAG_USE_GLOBAL) &&
+								 (((ircnet *)serv->network)->user)) ?
 								(((ircnet *)serv->network)->user) :
 								prefs.username,
-								(((ircnet *)serv->network)->real) ?
+								(!(((ircnet *)serv->network)->flags & FLAG_USE_GLOBAL) &&
+								 (((ircnet *)serv->network)->real)) ?
 								(((ircnet *)serv->network)->real) :
 								prefs.realname);
 		} else
@@ -805,7 +815,7 @@ waitline2 (GIOChannel *source, char *buf, int bufsize)
 
 #else
 
-#define waitline2(source,buf,size) waitline(serv->childread,buf,size)
+#define waitline2(source,buf,size) waitline(serv->childread,buf,size,0)
 
 #endif
 
@@ -860,6 +870,10 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 
 	switch (tbuf[0])
 	{
+	case '0':	/* print some text */
+		waitline2 (source, tbuf, sizeof tbuf);
+		PrintText (serv->server_session, tbuf);
+		break;
 	case '1':						  /* unknown host */
 		server_stopconnecting (serv);
 		closesocket (serv->sok4);
@@ -1241,7 +1255,35 @@ base64_encode (char *to, char *from, unsigned int len)
 }
 
 static int
-traverse_http (int sok, char *serverAddr, int port)
+http_read_line (int print_fd, int sok, char *buf, int len)
+{
+#ifdef WIN32
+	/* make sure waitline() uses recv() or it'll fail on win32 */
+	len = waitline (sok, buf, len, FALSE);
+#else
+	len = waitline (sok, buf, len, TRUE);
+#endif
+	if (len >= 1)
+	{
+		/* print the message out (send it to the parent process) */
+		write (print_fd, "0\n", 2);
+
+		if (buf[len-1] == '\r')
+		{
+			buf[len-1] = '\n';
+			write (print_fd, buf, len);
+		} else
+		{
+			write (print_fd, buf, len);
+			write (print_fd, "\n", 1);
+		}
+	}
+
+	return len;
+}
+
+static int
+traverse_http (int print_fd, int sok, char *serverAddr, int port)
 {
 	char buf[256];
 	char auth_data[128];
@@ -1260,27 +1302,24 @@ traverse_http (int sok, char *serverAddr, int port)
 	n += snprintf (buf+n, sizeof (buf)-n, "\r\n");
 	send (sok, buf, n, 0);
 
-	/* dont eat this, it's actually informative to display */
-#if 0
-	waitline (sok, buf, sizeof (buf)); /* FIXME: win32 cant read() sok */
+	n = http_read_line (print_fd, sok, buf, sizeof (buf));
 	/* "HTTP/1.0 200 OK" */
-	if (strlen (buf) < 12)
+	if (n < 12)
 		return 1;
 	if (memcmp (buf, "HTTP/", 5) || memcmp (buf + 9, "200", 3))
 		return 1;
-	for (;;)
+	while (1)
 	{
 		/* read until blank line */
-		waitline (sok, buf, sizeof (buf));
-		if (!buf[0] || (buf[0] == '\r' && !buf[1]))
+		n = http_read_line (print_fd, sok, buf, sizeof (buf));
+		if (n < 1 || (n == 1 && buf[0] == '\n'))
 			break;
 	}
-#endif
 	return 0;
 }
 
 static int
-traverse_proxy (int sok, char *ip, int port)
+traverse_proxy (int print_fd, int sok, char *ip, int port)
 {
 	switch (prefs.proxy_type)
 	{
@@ -1291,7 +1330,7 @@ traverse_proxy (int sok, char *ip, int port)
 	case 3:
 		return traverse_socks5 (sok, ip, port);
 	case 4:
-		return traverse_http (sok, ip, port);
+		return traverse_http (print_fd, sok, ip, port);
 	}
 
 	return 1;
@@ -1387,7 +1426,7 @@ server_child (server * serv)
 		/* connect succeeded */
 		if (proxy_ip)
 		{
-			switch (traverse_proxy (sok, proxy_ip, port))
+			switch (traverse_proxy (serv->childwrite, sok, proxy_ip, port))
 			{
 			case 0:	/* success */
 				snprintf (buf, sizeof (buf), "4\n%d\n", sok);	/* success */
@@ -1469,7 +1508,7 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	safe_strcpy (serv->servername, hostname, sizeof (serv->servername));
 	safe_strcpy (serv->hostname, hostname, sizeof (serv->hostname));
 
-	set_server_defaults (serv);
+	server_set_defaults (serv);
 	serv->connecting = TRUE;
 	serv->port = port;
 	serv->no_login = no_login;
@@ -1515,7 +1554,8 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 		fe_input_add (serv->childread, FIA_READ|FIA_FD, server_read_child, serv);
 }
 
-void server_fill_her_up (server *serv)
+void
+server_fill_her_up (server *serv)
 {
 	serv->connect = server_connect;
 	serv->disconnect = server_disconnect;
@@ -1525,3 +1565,96 @@ void server_fill_her_up (server *serv)
 
 	proto_fill_her_up (serv);
 }
+
+void
+server_set_encoding (server *serv, char *new_encoding)
+{
+	char *space;
+
+	if (serv->encoding)
+	{
+		free (serv->encoding);
+		/* can be left as NULL to indicate system encoding */
+		serv->encoding = NULL;
+		serv->using_cp1255 = FALSE;
+	}
+
+	if (new_encoding)
+	{
+		serv->encoding = strdup (new_encoding);
+		/* the serverlist GUI might have added a space 
+			and short description - remove it. */
+		space = strchr (serv->encoding, ' ');
+		if (space)
+			space[0] = 0;
+
+		/* server_inline() uses this flag */
+		if (!strcasecmp (serv->encoding, "CP1255") ||
+			 !strcasecmp (serv->encoding, "WINDOWS-1255"))
+			serv->using_cp1255 = TRUE;
+	}
+}
+
+server *
+server_new (void)
+{
+	static int id = 0;
+	server *serv;
+
+	serv = malloc (sizeof (struct server));
+	memset (serv, 0, sizeof (struct server));
+
+	/* use server.c and proto-irc.c functions */
+	server_fill_her_up (serv);
+
+	serv->id = id++;
+	serv->sok = -1;
+	strcpy (serv->nick, prefs.nick1);
+	server_set_defaults (serv);
+
+	serv_list = g_slist_prepend (serv_list, serv);
+
+	fe_new_server (serv);
+
+	return serv;
+}
+
+void
+server_set_defaults (server *serv)
+{
+	if (serv->chantypes)
+		free (serv->chantypes);
+	if (serv->chanmodes)
+		free (serv->chanmodes);
+	if (serv->nick_prefixes)
+		free (serv->nick_prefixes);
+	if (serv->nick_modes)
+		free (serv->nick_modes);
+
+	serv->chantypes = strdup ("#&!+");
+	serv->chanmodes = strdup ("beI,k,l");
+	serv->nick_prefixes = strdup ("@%+");
+	serv->nick_modes = strdup ("ohv");
+
+	serv->nickcount = 1;
+	serv->nickservtype = 0;
+	serv->end_of_motd = FALSE;
+	serv->is_away = FALSE;
+	serv->supports_watch = FALSE;
+	serv->bad_prefix = FALSE;
+	serv->use_who = TRUE;
+	serv->have_idmsg = FALSE;
+	serv->have_except = FALSE;
+}
+
+char *
+server_get_network (server *serv, gboolean fallback)
+{
+	if (serv->network)
+		return ((ircnet *)serv->network)->name;
+
+	if (fallback)
+		return serv->servername;
+
+	return NULL;
+}
