@@ -29,6 +29,7 @@
 #include "fe.h"
 #include "server.h"
 #include "text.h"
+#include "util.h" /* token_foreach */
 #include "xchatc.h"
 
 #include "servlist.h"
@@ -39,6 +40,7 @@ struct defaultserver
 	char *network;
 	char *host;
 	char *channel;
+	char *charset;
 };
 
 static const struct defaultserver def[] =
@@ -138,6 +140,10 @@ static const struct defaultserver def[] =
 	{0,			"unix.coolchat.net"},
 	{0,			"toronto.coolchat.net"},
 
+	{"Criten", 0},
+	{0,			"irc.criten.net"},
+	{0,			"irc.eu.criten.net"},
+
 	{"DALnet",	0},
 	{0,			"irc.dal.net"},
 	{0,			"irc.eu.dal.net"},
@@ -159,10 +165,12 @@ static const struct defaultserver def[] =
 	{0,			"irc.lagged.org"},
 	{0,			"irc.lightning.net"},
 	{0,			"irc.mindspring.com"},
-	{0,			"irc.rt.ru"},
 	{0,			"irc.easynews.com"},
 	{0,			"irc.servercentral.net"},
 
+	{"EnterTheGame",	0},
+	{0,			"IRC.EnterTheGame.Com"},
+
 	{"EUIrc",	0},
 	{0,			"irc.euirc.net"},
 	{0,			"irc.ham.de.euirc.net"},
@@ -244,22 +252,18 @@ static const struct defaultserver def[] =
 	{0,			"tonsberg.no.eu.irclink.net"},
 
 	{"IRCNet",		0},
-	{0,				"irc.stealth.net/5550"},
+	{0,				"irc.ircnet.com"},
+	{0,				"irc.stealth.net/6668"},
 	{0,				"ircnet.demon.co.uk"},
 	{0,				"ircnet.hinet.hr"},
 	{0,				"irc.datacomm.ch"},
 	{0,				"ircnet.kaptech.fr"},
-	{0,				"irc.flashnet.it"},
-	{0,				"irc.cwitaly.it"},
 	{0,				"ircnet.easynet.co.uk"},
 	{0,				"random.ircd.de"},
 	{0,				"ircnet.netvision.net.il"},
 	{0,				"irc.seed.net.tw"},
 	{0,				"irc.cs.hut.fi"},
 
-	{"IRCSoulZ",	0},
-	{0,			"irc.ircsoulz.net"},
-
 	{"Irctoo.net",	0},
 	{0,			"irc.canadian.net"},
 	{0,			"irc.irctoo.net"},
@@ -308,9 +312,6 @@ static const struct defaultserver def[] =
 	{"OFTC",	0},
 	{0,			"irc.oftc.net"},
 
-	{"OnlyIRC",		0},
-	{0,			"irc.onlyirc.net"},
-
 	{"OtherNet",	0},
 	{0,			"irc.othernet.org"},
 
@@ -318,12 +319,6 @@ static const struct defaultserver def[] =
 	{0,			"sydney.oz.org"},
 	{0,			"melbourne.oz.org"},
 
-	{"Progameplayer",	0},
-	{0,			"irc.progameplayer.com"},
-	{0,			"melancholia.oh.us.progameplayer.com"},
-	{0,			"deimos.oh.us.progameplayer.com"},
-	{0,			"paradigm.oh.us.progameplayer.com"},
-
 	{"PTlink",	0},
 	{0,			"irc.PTlink.net"},
 	{0,			"aaia.PTlink.net"},
@@ -359,9 +354,6 @@ static const struct defaultserver def[] =
 	{0,			"iscte.PTnet.org"},
 	{0,			"ubi.PTnet.org"},
 
-	{"QChat.net",	0},
-	{0,			"irc.qchat.net"},
-
 	{"QuakeNet",	0},
 	{0,			"irc.quakenet.org"},
 	{0,			"irc.se.quakenet.org"},
@@ -373,11 +365,6 @@ static const struct defaultserver def[] =
 	{0,			"irc.de.quakenet.org"},
 	{0,			"irc.it.quakenet.org"},
 
-	{"RealmNET",	0},
-	{0,			"irc.realmnet.com"},
-	{0,			"irc.eu.realmnet.com"},
-	{0,			"irc.us.realmnet.com"},
-
 	{"RebelChat",	0},
 	{0,			"irc.rebelchat.org"},
 
@@ -391,9 +378,6 @@ static const struct defaultserver def[] =
 	{0,			"garbage.recycled-irc.org"},
 	{0,			"dustbin.recycled-irc.org"},
 
-	{"RedeBrasilIRC",	0},
-	{0,			"irc.redebrasilirc.org"},
-
 	{"RizeNET", 0},
 	{0,			"irc.rizenet.org"},
 	{0,			"omega.rizenet.org"},
@@ -401,6 +385,13 @@ static const struct defaultserver def[] =
 	{0,			"lisa.rizenet.org"},
 	{0,			"scott.rizenet.org"},
 
+	{"RusNet", 0, 0, "KOI8-R"},
+	{0,			"irc.tomsk.net"},
+	{0,			"irc.rinet.ru"},
+	{0,			"irc.run.net"},
+	{0,			"irc.ru"},
+	{0,			"irc.lucky.net"},
+
 	{"SceneNet",	0},
 	{0,			"irc.scene.org"},
 	{0,			"irc.eu.scene.org"},
@@ -434,12 +425,6 @@ static const struct defaultserver def[] =
 	{0,			"tahoma.starchat.net"},
 	{0,			"neo.starchat.net"},
 
-	{"SubCultNet",	0},
-	{0,			"irc.subcult.ch"},
-	{0,			"irc.phuncrew.ch"},
-	{0,			"irc.mgz.ch"},
-	{0,			"irc.nazgul.ch/6666"},
-
 	{"TNI3",			0},
 	{0,			"irc.tni3.com"},
 
@@ -458,9 +443,6 @@ static const struct defaultserver def[] =
 	{0,			"irc.telecoms.bg"},
 	{0,			"irc.tu-varna.edu"},
 
-	{"Unsecurity",	0},
-	{0,			"irc.unsecurity.org"},
-
 	{"Worldnet",		0},
 	{0,			"irc.worldnet.net"},
 	{0,			"irc.fr.worldnet.net"},
@@ -492,7 +474,6 @@ servlist_connect (session *sess, ircnet *net)
 	ircserver *ircserv;
 	GSList *list;
 	char *port;
-	char *space;
 	server *serv;
 
 	if (!sess)
@@ -509,7 +490,10 @@ servlist_connect (session *sess, ircnet *net)
 	/* incase a protocol switch is added to the servlist gui */
 	server_fill_her_up (sess->server);
 
-	if (net->autojoin && !sess->server->recondelay_tag)
+	sess->willjoinchannel[0] = 0;
+	serv->password[0] = 0;
+
+	if (net->autojoin)
 		safe_strcpy (sess->willjoinchannel, net->autojoin,
 						 sizeof (sess->willjoinchannel));
 	if (net->pass)
@@ -553,20 +537,9 @@ servlist_connect (session *sess, ircnet *net)
 
 		*port = '/';
 	} else
-		serv->connect (serv, ircserv->hostname, 6667, FALSE);
+		serv->connect (serv, ircserv->hostname, -1, FALSE);
 
-	if (serv->encoding)
-	{
-		free (serv->encoding);
-		serv->encoding = NULL;
-	}
-	if (net->encoding)
-	{
-		serv->encoding = strdup (net->encoding);
-		space = strchr (serv->encoding, ' ');
-		if (space)
-			space[0] = 0;
-	}
+	server_set_encoding (serv, net->encoding);
 }
 
 int
@@ -838,6 +811,11 @@ servlist_load_defaults (void)
 #endif
 			if (def[i].channel)
 				net->autojoin = strdup (def[i].channel);
+			if (def[i].charset)
+			{
+				free (net->encoding);
+				net->encoding = strdup (def[i].charset);
+			}
 			if (!strcmp (def[i].network, "ChatJunkies"))
 				prefs.slist_select = j;
 			j++;
@@ -966,41 +944,6 @@ servlist_write_ccmd (char *str, void *fp)
 	return fprintf (fp, "C=%s\n", (str[0] == '/') ? str + 1 : str);
 }
 
-/* separates a string according to a 'sep' char, then calls the callback
-   function for each token found */
-
-int
-token_foreach (char *str, char sep,
-					int (*callback) (char *str, void *ud), void *ud)
-{
-	char t, *start = str;
-
-	while (1)
-	{
-		if (*str == sep || *str == 0)
-		{
-			t = *str;
-			*str = 0;
-			if (callback (start, ud) < 1)
-			{
-				*str = t;
-				return FALSE;
-			}
-			*str = t;
-
-			if (*str == 0)
-				break;
-			str++;
-			start = str;
-
-		} else
-		{
-			str += g_utf8_skip[*((unsigned char *)str)];
-		}
-	}
-
-	return TRUE;
-}
 
 int
 servlist_save (void)
