@@ -51,6 +51,17 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define FILEPATH_LEN_MAX MAXPATHLEN
 #endif
 
+/* force a 32bit CMP.L */
+#ifdef _MSC_VER
+#define CMPL(a, c0, c1, c2, c3) (*(guint32 *)(a) == (guint32)(c0 | (c1 << 8) | (c2 << 16) | (c3 << 24)))
+#else
+#define CMPL(a, c0, c1, c2, c3) (a == (guint32)(c0 | (c1 << 8) | (c2 << 16) | (c3 << 24)))
+#endif
+/* force a 16bit CMP.W */
+#define CMPW(a, c0, c1) (*(guint16 *)(a) == (guint16)(c0 | (c1 << 8)))
+#define WORDL(c0, c1, c2, c3) (guint32)(c0 | (c1 << 8) | (c2 << 16) | (c3 << 24))
+#define WORDW(c0, c1) (guint16)(c0 | (c1 << 8))
+
 #ifdef WIN32						/* for win32 */
 #define OFLAGS O_BINARY
 #define sleep(t) _sleep(t*1000)
@@ -73,9 +84,10 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define NICKLEN		64				/* including the NULL, so 63 really */
 #define CHANLEN		300
 #define PDIWORDS		32
+#define USERNAMELEN 10
 
-#define safe_strcpy(dest,src,len)	{strncpy(dest,src,len); \
-												dest[len-1] = 0;}
+#define safe_strcpy(dest,src,len)	{strncpy((dest),(src),(len)); \
+												(dest)[len-1] = 0;}
 
 #if defined(ENABLE_NLS) && !defined(_)
 #  include <libintl.h>
@@ -114,16 +126,15 @@ struct xchatprefs
 	char quitreason[256];
 	char partreason[256];
 	char font_normal[FONTNAMELEN + 1];
-#ifdef USE_ZVT
-	char font_shell[FONTNAMELEN + 1];
-#endif
 	char doubleclickuser[256];
 	char sounddir[PATHLEN + 1];
 	char soundcmd[PATHLEN + 1];
 	char background[PATHLEN + 1];
 	char dccdir[PATHLEN + 1];
 	char dcc_completed_dir[PATHLEN + 1];
-	char bluestring[300];
+	char irc_extra_hilight[300];
+	char irc_no_hilight[300];
+	char irc_nick_hilight[300];
 	char dnsprogram[72];
 	char hostname[127];
 	char cmdchar[4];
@@ -240,6 +251,7 @@ struct xchatprefs
 	unsigned int auto_unmark_away;
 	unsigned int away_track;
 	unsigned int userhost;
+	unsigned int irc_whois_front;
 	unsigned int use_server_tab;
 	unsigned int notices_tabs;
 	unsigned int style_namelistgad;
@@ -293,8 +305,8 @@ typedef struct session
 	void *usertree;					/* ordered with Ops first */
 	struct User *me;					/* points to myself in the usertree */
 	char channel[CHANLEN];
-	char waitchannel[CHANLEN];		  /* waiting to join this channel */
-	char willjoinchannel[CHANLEN];	  /* /join done for this channel */
+	char waitchannel[CHANLEN];		  /* waiting to join channel (/join sent) */
+	char willjoinchannel[CHANLEN];	  /* will issue /join for this channel */
 	char channelkey[64];			  /* XXX correct max length? */
 	int limit;						  /* channel user limit */
 	int logfd;
@@ -453,6 +465,7 @@ typedef struct server
 	unsigned int have_capab:1;	/* supports CAPAB (005 tells us) */
 	unsigned int have_idmsg:1;	/* freenode's IDENTIFY-MSG */
 	unsigned int have_except:1;	/* ban exemptions +e */
+	unsigned int using_cp1255:1;	/* encoding is CP1255/WINDOWS-1255? */
 	int use_who:1;				/* whether to use WHO command to get dcc_ip */
 #ifdef USE_OPENSSL
 	int use_ssl:1;					  /* is server SSL capable? */
@@ -469,6 +482,7 @@ struct commands
 	cmd_callback callback;
 	char needserver;
 	char needchannel;
+	char handle_quotes;
 	char *help;
 };
 
