@@ -44,12 +44,12 @@
 #include "../common/xchat.h"
 #include "../common/fe.h"
 #include "../common/server.h"
-#include "../common/text.h"
 #include "../common/xchatc.h"
 #include "../common/outbound.h"
 #include "../common/inbound.h"
 #include "../common/plugin.h"
 #include "../common/modes.h"
+#include "../common/url.h"
 #include "fe-gtk.h"
 #include "banlist.h"
 #include "gtkutil.h"
@@ -164,6 +164,39 @@ flash_window (GtkWidget *win)
 	flash (&fi);
 	/*FlashWindowEx (&fi);*/
 }
+#else
+
+#ifdef USE_XLIB
+#include <gdk/gdkx.h>
+
+static void
+set_window_urgency (GtkWidget *win, gboolean set)
+{
+	XWMHints *hints;
+
+	hints = XGetWMHints(GDK_WINDOW_XDISPLAY(win->window), GDK_WINDOW_XWINDOW(win->window));
+	if (set)
+		hints->flags |= XUrgencyHint;
+	else
+		hints->flags &= ~XUrgencyHint;
+	XSetWMHints(GDK_WINDOW_XDISPLAY(win->window),
+	            GDK_WINDOW_XWINDOW(win->window), hints);
+	XFree(hints);
+}
+
+static void
+flash_window (GtkWidget *win)
+{
+	if (!gtk_window_has_toplevel_focus (GTK_WINDOW (win)))
+		set_window_urgency (win, TRUE);
+}
+
+static void
+unflash_window (GtkWidget *win)
+{
+	set_window_urgency (win, FALSE);
+}
+#endif
 #endif
 
 /* set a tab plain, red, light-red, or blue */
@@ -202,7 +235,7 @@ fe_set_tab_color (struct session *sess, int col, int flash)
 		}
 	}
 
-#ifdef WIN32
+#if defined(WIN32) || defined(USE_XLIB)
 	if (flash && prefs.flash_hilight)
 		flash_window (sess->gui->window);
 #endif
@@ -421,7 +454,7 @@ mg_focus (session *sess)
 	/* dirty trick to avoid auto-selection */
 	GTK_ENTRY (sess->gui->input_box)->editable = 0;
 	gtk_widget_grab_focus (sess->gui->input_box);
-	GTK_ENTRY (sess->gui->input_box)->editable = 1;
+	gtk_editable_set_editable (GTK_EDITABLE (sess->gui->input_box), TRUE);
 
 	sess->server->front_session = sess;
 
@@ -582,7 +615,7 @@ mg_set_topic_tip (session *sess)
 			 GTK_ENTRY (sess->gui->topic_entry)->text[0])
 			add_tip (sess->gui->topic_entry, GTK_ENTRY (sess->gui->topic_entry)->text);
 		else
-			add_tip (sess->gui->topic_entry, ""); /* hSP: set topic to empty in case it isn't set.. */
+			add_tip (sess->gui->topic_entry, NULL);
 	}
 }
 
@@ -634,6 +667,8 @@ mg_populate (session *sess)
 		gtk_widget_hide (gui->topicbutton_box);
 		/* hide the userlist */
 		mg_userlist_showhide (sess, FALSE);
+		/* shouldn't edit the topic */
+		gtk_editable_set_editable (GTK_EDITABLE (gui->topic_entry), FALSE);
 		break;
 	case SESS_SERVER:
 		if (prefs.chanmodebuttons)
@@ -642,6 +677,8 @@ mg_populate (session *sess)
 		gtk_widget_hide (gui->dialogbutton_box);
 		/* hide the userlist */
 		mg_userlist_showhide (sess, FALSE);
+		/* shouldn't edit the topic */
+		gtk_editable_set_editable (GTK_EDITABLE (gui->topic_entry), FALSE);
 		break;
 	default:
 		/* hide the dialog buttons */
@@ -650,6 +687,8 @@ mg_populate (session *sess)
 			gtk_widget_show (gui->topicbutton_box);
 		/* show the userlist */
 		mg_userlist_showhide (sess, TRUE);
+		/* let the topic be editted */
+		gtk_editable_set_editable (GTK_EDITABLE (gui->topic_entry), TRUE);
 	}
 
 	/* move to THE irc tab */
@@ -838,14 +877,14 @@ mg_ircdestroy (GtkWidget *tab, session *sess)
 static void
 mg_tabdestroy_cb (GtkWidget *tab, session *sess)
 {
-	if (tab == active_tab && !xchat_is_quitting)
+	if (tab == active_tab && mg_gui)
 		mg_find_replacement_focus (tab);
 
 	if (sess == NULL)
 		mg_gendestroy (tab);
 	else
 	{
-		if (!xchat_is_quitting)
+		if (mg_gui)	/* NULL if the last tab was just killed via WM */
 			tab_group_cleanup (sess->gui->tabs_box);
 		mg_ircdestroy (tab, sess);
 	}
@@ -904,7 +943,7 @@ mg_create_sess_tree (GtkWidget *menu)
 		if (serv->servername[0] == 0)
 			top_item = gtk_menu_item_new_with_label (_("<none>"));
 		else
-			top_item = gtk_menu_item_new_with_label (get_network (serv->front_session, TRUE));
+			top_item = gtk_menu_item_new_with_label (server_get_network (serv, TRUE));
 
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), top_item);
 		gtk_widget_show (top_item);
@@ -1078,9 +1117,6 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 		gtk_widget_show (item);
 	}
 
-	if (sess)
-		mg_create_color_menu (menu, sess);
-
 	item = gtk_menu_item_new_with_label (_("Go to"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -1109,12 +1145,61 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 	return TRUE;
 }
 
+void
+mg_dnd_drop_file (session *sess, char *target, char *uri)
+{
+	char *p, *data, *next, *fname;
+
+	p = data = strdup (uri);
+	while (*p)
+	{
+		next = strchr (p, '\r');
+		if (strncasecmp ("file:", p, 5) == 0)
+		{
+			if (next)
+				*next = 0;
+			fname = g_filename_from_uri (p, NULL, NULL);
+			if (fname)
+			{
+				/* dcc_send() expects utf-8 */
+				p = g_filename_to_utf8 (fname, -1, 0, 0, 0);
+				if (p)
+				{
+					dcc_send (sess, target, p, prefs.dcc_max_send_cps, 0);
+					g_free (p);
+				}
+				g_free (fname);
+			}
+		}
+		if (!next)
+			break;
+		p = next + 1;
+		if (*p == '\n')
+			p++;
+	}
+	free (data);
+
+}
+
+static void
+mg_dialog_dnd_drop (GtkWidget * widget, GdkDragContext * context, gint x,
+						  gint y, GtkSelectionData * selection_data, guint info,
+						  guint32 time, session *sess)
+{
+	/* sess->channel is really the nickname of dialogs */
+	mg_dnd_drop_file (sess, sess->channel, selection_data->data);
+}
+
 /* add a tabbed channel */
 
 static void
 mg_add_chan (session *sess)
 {
 	char *name = _("<none>");
+	static const GtkTargetEntry dnd_targets[] =
+	{
+		{"text/uri-list", 0, 1}
+	};
 
 	if (sess->channel[0])
 		name = sess->channel;
@@ -1137,6 +1222,16 @@ mg_add_chan (session *sess)
 		gtk_xtext_set_time_stamp (sess->res->buffer, prefs.timestamp);
 		sess->res->user_model = userlist_create_model ();
 	}
+
+#ifndef WIN32	/* needs more work */
+	if (sess->type == SESS_DIALOG)
+	{
+		gtk_drag_dest_set (sess->gui->xtext, GTK_DEST_DEFAULT_ALL, dnd_targets,
+								 1, GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
+		g_signal_connect (G_OBJECT (sess->gui->xtext), "drag_data_received",
+								G_CALLBACK (mg_dialog_dnd_drop), sess);
+	}
+#endif
 }
 
 static void
@@ -1240,7 +1335,7 @@ mg_topic_cb (GtkWidget *entry, gpointer userdata)
 	session *sess = current_sess;
 	char *text;
 
-	if (sess->channel[0] && sess->server->connected)
+	if (sess->channel[0] && sess->server->connected && sess->type == SESS_CHANNEL)
 	{
 		text = GTK_ENTRY (entry)->text;
 		if (text[0] == 0)
@@ -1271,7 +1366,8 @@ mg_dcc_active (void)
 	while (list)
 	{
 		dcc = list->data;
-		if (dcc->dccstat == STAT_ACTIVE)
+		if ((dcc->type == TYPE_SEND || dcc->type == TYPE_RECV) &&
+			 dcc->dccstat == STAT_ACTIVE)
 			return 1;
 		list = list->next;
 	}
@@ -1633,7 +1729,7 @@ mg_dialog_button_cb (GtkWidget *wid, char *cmd)
 	/* dirty trick to avoid auto-selection */
 	GTK_ENTRY (current_sess->gui->input_box)->editable = 0;
 	gtk_widget_grab_focus (current_sess->gui->input_box);
-	GTK_ENTRY (current_sess->gui->input_box)->editable = 1;
+	gtk_editable_set_editable (GTK_EDITABLE (current_sess->gui->input_box), TRUE);
 }
 
 static void
@@ -1706,18 +1802,30 @@ mg_create_topicbar (session *sess, GtkWidget *box, char *name)
 /* check if a word is clickable */
 
 static int
-mg_word_check (GtkWidget * xtext, char *word)
+mg_word_check (GtkWidget * xtext, char *word, int len)
 {
-	return text_word_check (word);	/* common/text.c */
+	session *sess = current_sess;
+	int ret;
+
+	ret = url_check_word (word, len);	/* common/url.c */
+	if (ret == 0)
+	{
+		if (( (word[0]=='@' || word[0]=='+') && find_name (sess, word+1)) || find_name (sess, word))
+			return WORD_NICK;
+
+		if (sess->type == SESS_DIALOG)
+			return WORD_DIALOG;
+	}
+
+	return ret;
 }
 
 /* mouse click inside text area */
 
 static void
-mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
-					  session *sess)
+mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even)
 {
-	sess = current_sess;
+	session *sess = current_sess;
 
 	if (even->button == 1)			/* left button */
 	{
@@ -1729,7 +1837,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 
 		if (even->state & GDK_CONTROL_MASK)
 		{
-			switch (mg_word_check (xtext, word))
+			switch (mg_word_check (xtext, word, strlen (word)))
 			{
 			case WORD_URL:
 			case WORD_HOST:
@@ -1748,7 +1856,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 		return;
 	}
 
-	switch (mg_word_check (xtext, word))
+	switch (mg_word_check (xtext, word, strlen (word)))
 	{
 	case 0:
 		menu_middlemenu (sess, even);
@@ -2068,6 +2176,12 @@ mg_set_tabs_pos (session_gui *gui, int pos)
 		gtk_widget_unref (gui->tabs_box);
 }
 
+static void
+mg_inputbox_rightclick (GtkEntry *entry, GtkWidget *menu)
+{
+	mg_create_color_menu (menu, NULL);
+}
+
 static void
 mg_create_entry (session *sess, GtkWidget *box)
 {
@@ -2096,6 +2210,8 @@ mg_create_entry (session *sess, GtkWidget *box)
 							G_CALLBACK (mg_inputbox_focus), gui);
 	g_signal_connect (G_OBJECT (entry), "activate",
 							G_CALLBACK (mg_inputbox_cb), gui);
+	g_signal_connect (G_OBJECT (entry), "populate_popup",
+							G_CALLBACK (mg_inputbox_rightclick), NULL);
 	gtk_container_add (GTK_CONTAINER (hbox), entry);
 	gtk_widget_grab_focus (entry);
 
@@ -2171,6 +2287,11 @@ mg_tabwin_focus_cb (GtkWindow * win, GdkEventFocus *event, gpointer userdata)
 		gtk_xtext_check_marker_visibility (GTK_XTEXT (current_sess->gui->xtext));
 		plugin_emit_dummy_print (current_sess, "Focus Window");
 	}
+#ifndef WIN32
+#ifdef USE_XLIB
+	unflash_window (GTK_WIDGET (win));
+#endif
+#endif
 	return FALSE;
 }
 
@@ -2181,6 +2302,11 @@ mg_topwin_focus_cb (GtkWindow * win, GdkEventFocus *event, session *sess)
 	if (!sess->server->server_session)
 		sess->server->server_session = sess;
 	gtk_xtext_check_marker_visibility(GTK_XTEXT (current_sess->gui->xtext));
+#ifndef WIN32
+#ifdef USE_XLIB
+	unflash_window (GTK_WIDGET (win));
+#endif
+#endif
 	plugin_emit_dummy_print (sess, "Focus Window");
 	return FALSE;
 }
@@ -2791,7 +2917,7 @@ fe_session_callback (session *sess)
 	if (sess->gui->bartag)
 		fe_timeout_remove (sess->gui->bartag);
 
-	if (sess->gui != mg_gui)
+	if (sess->gui != &static_mg_gui)
 		free (sess->gui);
 	free (sess->res);
 }
